@@ -1,15 +1,21 @@
+/*
+ * On WIN32 windows.h and winsock.h need to be included before mysql.h
+ * Otherwise SOCKET type which is needed for mysql.h is not defined
+ */
 #ifdef _WIN32
-#include "windows.h"
-#include "winsock.h"
+#include <windows.h>
+#include <winsock.h>
 #endif
+#include <mysql.h>
+#include <stddef.h>
+#include <errno.h>
 
 #ifndef _WIN32
-#include <poll.h>
-#include <errno.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
 #endif
 
-#include <mysql.h>
-
 /*
  * Warning: Native socket code must be outside of dbdimp.c and dbdimp.h because
  *          perl header files redefine socket function. This file must not
@@ -18,12 +24,9 @@
 
 int mariadb_dr_socket_ready(my_socket fd)
 {
-  int retval;
-
-#ifdef _WIN32
-  /* Windows does not have poll(), so use select() instead */
   struct timeval timeout;
   fd_set fds;
+  int retval;
 
   FD_ZERO(&fds);
   FD_SET(fd, &fds);
@@ -32,15 +35,6 @@ int mariadb_dr_socket_ready(my_socket fd)
   timeout.tv_usec = 0;
 
   retval = select(fd+1, &fds, NULL, NULL, &timeout);
-#else
-  struct pollfd fds;
-
-  fds.fd = fd;
-  fds.events = POLLIN;
-
-  retval = poll(&fds, 1, 0);
-#endif
-
   if (retval < 0) {
 #ifdef _WIN32
     /* Windows does not update errno */
